package com.landray.kmss.sys.attachment.util;

import javax.imageio.stream.ImageOutputStream;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.IIOImage;
import javax.imageio.ImageWriteParam;
import java.awt.image.RenderedImage;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.plugins.jpeg.JPEGImageWriteParam;
import javax.imageio.ImageWriter;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.awt.Image;
import java.awt.image.ImageObserver;
import javax.imageio.ImageIO;
import java.io.OutputStream;
import java.io.InputStream;
import com.landray.kmss.util.StringUtil;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;

public class ImageCompressUtils
{
    public static final String IMG_TYPE_JPEG = "jpeg";
    public static final List<String> IMG_TYPES;
    private static ImageCompressUtils instance;
    
    static {
        (IMG_TYPES = new ArrayList<String>()).add("jpg");
        ImageCompressUtils.IMG_TYPES.add("jpeg");
        ImageCompressUtils.IMG_TYPES.add("bmp");
        ImageCompressUtils.IMG_TYPES.add("gif");
        ImageCompressUtils.IMG_TYPES.add("png");
        ImageCompressUtils.IMG_TYPES.add("tif");
        ImageCompressUtils.instance = new ImageCompressUtils();
    }
    
    public static final boolean isImageType(String type) {
        type = type.toLowerCase();
        for (final String imgType : ImageCompressUtils.IMG_TYPES) {
            if (imgType.equals(type)) {
                return true;
            }
        }
        return false;
    }
    
    protected ImageCompressUtils() {
    }
    
    public static ImageCompressUtils getInstance() {
        return ImageCompressUtils.instance;
    }
    
    public final boolean isBMP(final String fileName) {
        return !StringUtil.isNull(fileName) && fileName.toLowerCase().lastIndexOf(".bmp") == fileName.length() - 4;
    }
    
    public void compressImage(final InputStream src, final OutputStream out, final int width, final int height, final boolean proportion) throws IOException {
        final Image img = ImageIO.read(src);
        final int imgW = img.getWidth(null);
        final int imgH = img.getHeight(null);
        if (width >= imgW && height >= imgH) {
            return;
        }
        this.compressImage(img, out, width, height, proportion);
    }
    
    public InputStream compressImage(final InputStream src, final int width, final int height, final boolean proportion) throws IOException {
        final Image img = ImageIO.read(src);
        final int imgW = img.getWidth(null);
        final int imgH = img.getHeight(null);
        if (width >= imgW && height >= imgH) {
            return src;
        }
        final ByteArrayOutputStream out = new ByteArrayOutputStream();
        this.compressImage(img, out, width, height, proportion);
        return new ByteArrayInputStream(out.toByteArray());
    }
    
    public void compressImage(final Image src, final OutputStream out, final int width, final int height, final boolean proportion) throws IOException {
        final Image img = src;
        int newWidth;
        int newHeight;
        if (proportion) {
            final double imgW = img.getWidth(null);
            final double imgH = img.getHeight(null);
            final double rate1 = imgW / width + 0.1;
            final double rate2 = imgH / height + 0.1;
            final double rate3 = (rate1 > rate2) ? rate1 : rate2;
            newWidth = (int)(imgW / rate3);
            newHeight = (int)(imgH / rate3);
        }
        else {
            newWidth = width;
            newHeight = height;
        }
        this.compressImage(img, out, newWidth, newHeight, "jpeg");
    }
    
    public void compressImage(final Image img, final OutputStream out, final int width, final int height, final String type) throws IOException {
        final BufferedImage tag = new BufferedImage(width, height, 1);
        tag.getGraphics().drawImage(img.getScaledInstance(width, height, 4), 0, 0, null);
        final ImageWriter imageWriter = ImageIO.getImageWritersBySuffix(type).next();
        final ImageOutputStream ios = ImageIO.createImageOutputStream(out);
        imageWriter.setOutput(ios);
        final JPEGImageWriteParam jpegParams = (JPEGImageWriteParam)imageWriter.getDefaultWriteParam();
        jpegParams.setCompressionMode(2);
        jpegParams.setCompressionQuality(1.0f);
        final IIOMetadata imageMetaData = imageWriter.getDefaultImageMetadata(new ImageTypeSpecifier(tag), jpegParams);
        imageWriter.write(imageMetaData, new IIOImage(tag, null, null), jpegParams);
        ios.close();
        imageWriter.dispose();
    }
}
